function signalred = classifysignals(ic,signal,des)
%CLASSIFYSINGALS Extracts ECG, BP/PPG and SV.
%
% IN
%
%    ic     - n x t matrix with n independent components of t samples each.
%    signal - m x t matrix with m signals of t samples each.
%    des    - m x 1 cell that describes each sensor in signal.
%
% OUT
%
%    signalred - 3 x t matrix containing
%
%       signalred(1,:) - ECG from ic.
%       signalred(2,:) - BP/PPG from signal if it exists, otherwise this
%                        row is filled with NaNs.
%       signalred(3,:) - SV from signal if it exists, otherwise this row is
%                        filled with NaNs.

des = cellfun(@lower,des,'UniformOutput',0);
signalred = zeros(3,size(ic,2));

%% Locate ECG from ICA

% Extract correct trace from ic inputs
[reqTraceEntry,probTraceEntry] = locateCorrectTrace(ic,1);

% Check whether probability is above 60%; else take 'des' entry
if isempty(reqTraceEntry{1}) || (~isempty(reqTraceEntry{1}) && probTraceEntry{1} < 60)
    desEntry = ~cellfun(@isempty,(strfind(des,'ecg')));
    reqTraceEntry{1} = find(desEntry,1,'first');
    signalred(1,:) = signal(reqTraceEntry{1},:);
    display(['ECG not found! p = ' num2str(0.01*round(100*probTraceEntry{1}))])
else
    signalred(1,:) = ic(reqTraceEntry{1},:);
    display(['ECG found! p = ' num2str(0.01*round(100*probTraceEntry{1}))])
end

%% Locate BP/PPG and SV

% Extract correct traces BP/PPF and SV from signal inputs
reqTrace = [2,3];
[reqTraceEntry,probTraceEntry] = locateCorrectTrace(signal,reqTrace);

% Check whether BP/PPG probability is above 60%; else take 'des' entry if present
if isempty(reqTraceEntry{1}) || (~isempty(reqTraceEntry{1}) && probTraceEntry{1} < 60)
    desEntry = ~cellfun(@isempty,(strfind(des,'bp')));
    if (~any(desEntry))
        desEntry = ~cellfun(@isempty,(strfind(des,'ppg')));
    end
    reqTraceEntry{1} = find(desEntry,1,'first');
    display(['BP/PPG not found! p = ' num2str(0.01*round(100*probTraceEntry{1}))])
else
    display(['BP/PPG found! p = ' num2str(0.01*round(100*probTraceEntry{1}))])
end

% Fill in BP/PPG in signalred
if ~isempty(reqTraceEntry{1})
    signalred(2,:) = signal(reqTraceEntry{1},:);
else
    signalred(2,:) = nan(1,size(signalred(2,:),2));
end

% Check whether SV probability is above 60%; else take 'des' entry if present
if isempty(reqTraceEntry{2}) || (~isempty(reqTraceEntry{2}) && probTraceEntry{2} < 60)
    desEntry = ~cellfun(@isempty,(strfind(des,'sv')));
    reqTraceEntry{2} = find(desEntry,1,'first');
    display(['SV not found! p = ' num2str(0.01*round(100*probTraceEntry{2}))])
else
    display(['SV found! p = ' num2str(0.01*round(100*probTraceEntry{2}))])
end

% Fill in SV in signalred
if ~isempty(reqTraceEntry{2})
    signalred(3,:) = signal(reqTraceEntry{2},:);
else
    signalred(3,:) = nan(1,size(signalred(3,:),2));
end

end

function traceFeatures = calcFeatures(trace)

trace = trace';

% Detrend data over 5 seconds
trace = detrendmedian(trace,5*250,50,10); % 5 second window @ 250 Hz

% Normalize detrended data
trace = trace/prctile(abs(trace),99);

% Calculate 2.5, 10, 20, 30, 40, 60, 70, 80, 90, 97.5 percentiles
prctileTrace = prctile(abs(trace),[2.5,10,20,30,40,60,70,80,90,97.5]);

% Calculate frequency spectrum
[energy,frequency] = calcFFT(trace,250); % 250 Hz

% Calculate normalized energy in bands 0.1-0.5, 0.5:1.5, 1.5:3, 3:5, 5:8, 9:12, 12:16, 16:20
totalEnergy = sum(energy(frequency >= 0.1 & frequency <= 20));
energyBand = zeros(1,8);
energyBand(1) = sum(energy(frequency >= 0.1 & frequency <= 0.5));
energyBand(2) = sum(energy(frequency >= 0.5 & frequency <= 1.5));
energyBand(3) = sum(energy(frequency >= 1.5 & frequency <= 3));
energyBand(4) = sum(energy(frequency >= 3 & frequency <= 5));
energyBand(5) = sum(energy(frequency >= 5 & frequency <= 8));
energyBand(6) = sum(energy(frequency >= 8 & frequency <= 12));
energyBand(7) = sum(energy(frequency >= 12 & frequency <= 16));
energyBand(8) = sum(energy(frequency >= 16 & frequency <= 20));
energyBand = energyBand/totalEnergy;

% Fill in features in featureMatrix
traceFeatures = [prctileTrace(:)',energyBand];

end

function [reqTraceEntry,probTraceEntry] = locateCorrectTrace(inputTraces,reqTrace)

load traceDetectionModelParameters;

% Calculate features
traceFeaturesNorm = zeros(size(inputTraces,1),18);
for i = 1:size(inputTraces,1)
    traceFeatures = calcFeatures(inputTraces(i,:));
    traceFeaturesNorm(i,:) = (traceFeatures-repmat(featureShift,size(traceFeatures,1),1))*spdiags(featureScale',0,size(traceFeatures,2),size(traceFeatures,2));
end

% Calculate Ylatent with multiclass classifier
[~,Ylatent] = simlssvm(model,traceFeaturesNorm);

% Transform output to summed probability for every possibility
prob = zeros(size(Ylatent,1),8);
for n = 1:size(Ylatent,1)
    for i = 1:8
        for j = 1:28
            if (~isnan(model.codebook2(j,i)))
                prob(n,i) = prob(n,i)+50+sign(model.codebook2(j,i))*sign(Ylatent(n,j)).*(50-50*exp(-abs(Ylatent(n,j))*-log(0.2)));
            end
        end
    end
end

% Locate two most probable signal types, and take class and probability
% between these two
probSignal = zeros(size(prob,1),2);
for n = 1:size(prob,1)
    [~,sortedIndex] = sort(prob(n,:),2,'descend');
    codebook2entry = sum([0,7:-1:(9-min(sortedIndex(1:2)))]) + (max(sortedIndex(1:2))-min(sortedIndex(1:2)));
    switch sign(Ylatent(n,codebook2entry))
        case 1
            probSignal(n,1) = min(sortedIndex(1:2));
        case -1
            probSignal(n,1) = max(sortedIndex(1:2));
    end
    probSignal(n,2) = 50+(50-50*exp(-abs(Ylatent(n,codebook2entry))*-log(0.2)));
end

% Locate required traces (or most probable ones)
reqTraceEntry = cell(1,length(reqTrace));
probTraceEntry = cell(1,length(reqTrace));
for i = 1:length(reqTraceEntry)
    reqTraceEntries = find(probSignal==reqTrace(i));
    reqTraceEntry{i} = reqTraceEntries(maxIndex(probSignal(reqTraceEntries,2),1));
    probTraceEntry{i} = probSignal(reqTraceEntry{i},2);
end

end

function [I] = maxIndex(X,dim)

% Function that returns the index of the maximum value of the array or
% matrix X, following the input dimension. In case of multiple values
% having the same maximum value, the lowest index value is returned.
%
% Use: [I] = maxIndex(X,dim)
%
% Author: Tim Willemen
% Last edit: 04-04-2013

[~,I] = max(X,[],dim);

end

function [I] = minIndex(X,dim)

% Function that returns the index of the minimum value of the array or
% matrix X, following the input dimension. In case of multiple values
% having the same minimum value, the lowest index value is returned.
%
% Use: [I] = minIndex(X,dim)
%
% Author: Tim Willemen
% Last edit: 04-04-2013

[~,I] = min(X,[],dim);

end

function [energy frequency] = calcFFT(y,sampleRate)

% Function that calculates the energy spectrum via the Fast Fourier
% Transform algorithm, for a signal with a sample rate defined by the
% input 'sampleRate'. The signal is automatically padded with zeros to have
% its length equal to a power of 2. A hanning window is also applied to the
% signal to minimize spectral leakage.
%
% Use: [energy frequency] = calcFFT(y,sampleRate)
% 
% Author: Tim Willemen
% Last edit: 27-03-2013

L = length(y);
NFFT = 2^nextpow2(L); % Next power of 2
Y = fft(y.*hann(length(y)),NFFT)/L; % Rescale coefficients back to original signal length by dividing by L
f = (sampleRate/2*linspace(0,1,NFFT/2+1))';

energy = (2*abs(Y(1:NFFT/2+1))).^2; % times two, since we cut signal in half
frequency = f;

end
