function [HRtimes] = hrfilter(HRtimes)

% Automatic filtering of too short and too long RR's, adapted from De Chazal et al, 2003.
%
% Use: [HRtimes] = hrFilter(HRtimes)
% 
% Author: Tim Willemen
% Last edit: 18-03-2014

HR = diff(HRtimes);

%% Correction of too short intervals

% Detection of too short RR intervals by comparing them with the median
% value of the surrounding 5 RR intervals. Intervals being too short will
% be added to the next or previous RR interval if this makes them closer to
% the median value. We rerun this script until no too short RR intervals
% are detected anymore.
tooShortIntervalsPresent = true;

while(tooShortIntervalsPresent)

    % Defining robust RR series
    HRrobust = runprctile(HR,2,50);

    % Defining new RR series by always adding next or previous RR interval to
    % current interval.
    HRaddPrevious = zeros(size(HR));
    HRaddPrevious(1) = 2*HR(1);
    HRaddPrevious(2:end) = HR(1:end-1)+HR(2:end);
    HRaddNext = zeros(size(HR));
    HRaddNext(end) = 2*HR(end);
    HRaddNext(1:end-1) = HR(1:end-1)+HR(2:end);

    % Detecting for which intervals adding the next or previous RR interval
    % makes it closer to the robust RR interval value.
    HRminusRobust = HR-HRrobust;
    HRaddPreviousMinusRobust = HRaddPrevious-HRrobust;
    HRaddNextMinusRobust = HRaddNext-HRrobust;
    temp = minIndex(abs([HRminusRobust HRaddPreviousMinusRobust HRaddNextMinusRobust]),2);
    tooShortRR = find(temp~=1);
    try
        if (tooShortRR(1) == 1), tooShortRR = tooShortRR(2:end); end
        if (tooShortRR(end) == length(HR)), tooShortRR = tooShortRR(1:end-1); end
    catch ME
        % no elements left in tooShortRR
    end
    
    if (~isempty(tooShortRR))

        % Looping over all to be corrected RR intervals.
        correctedHR = HR;
        for i = 1:length(tooShortRR)
            if (correctedHR(tooShortRR(i)) ~= 9999);
                if (temp(tooShortRR(i)) == 3)
                    firstNon9999 = find(correctedHR(tooShortRR(i)+1:end)~=9999,1,'first');
                    correctedHR(tooShortRR(i)) = correctedHR(tooShortRR(i)) + correctedHR(tooShortRR(i)+firstNon9999);
                    if (tooShortRR(i)+firstNon9999 <= length(correctedHR))
                        correctedHR(tooShortRR(i)+firstNon9999) = 9999;
                    end
                elseif (temp(tooShortRR(i)) == 2)
                    lastNon9999 = find(correctedHR(1:tooShortRR(i)-1)~=9999,1,'last');
                    correctedHR(tooShortRR(i)) = correctedHR(tooShortRR(i)) + correctedHR(lastNon9999);
                    if (lastNon9999 >= 1)
                        correctedHR(lastNon9999) = 9999;
                    end
                end
            end
        end
        % Redefining HR and HRtimes
        HR = correctedHR(correctedHR~=9999);
    else
        % End while loop
        tooShortIntervalsPresent = false;
    end
end

% Detecting intervals shorter than 0.30 seconds, as being a lower physiological limit. We
% also rerun this script until no too short RR intervals are detected anymore.
tooShortIntervalsPresent = true;

while(tooShortIntervalsPresent)

    tooShortRR = find(HR<0.30);
    try
        if (tooShortRR(1) == 1), tooShortRR = tooShortRR(2:end); end
        if (tooShortRR(end) == length(HR)), tooShortRR = tooShortRR(1:end-1); end
    catch ME
        % no elements left in tooShortRR
    end
    
    if (~isempty(tooShortRR))

        % Looping over all to be corrected RR intervals.
        HRaddPrevious = zeros(size(HR));
        HRaddPrevious(1) = 2*HR(1);
        HRaddPrevious(2:end) = HR(1:end-1)+HR(2:end);
        HRaddNext = zeros(size(HR));
        HRaddNext(end) = 2*HR(end);
        HRaddNext(1:end-1) = HR(1:end-1)+HR(2:end);
        temp = minIndex([HRaddPrevious HRaddNext],2);

        correctedHR = HR;
        for i = 1:length(tooShortRR)
            if (correctedHR(tooShortRR(i)) ~= 9999);
                if (temp(tooShortRR(i)) == 2) % Add next
                    firstNon9999 = find(correctedHR(tooShortRR(i)+1:end)~=9999,1,'first');
                    correctedHR(tooShortRR(i)) = correctedHR(tooShortRR(i)) + correctedHR(tooShortRR(i)+firstNon9999);
                    if (tooShortRR(i)+firstNon9999 <= length(correctedHR))
                        correctedHR(tooShortRR(i)+firstNon9999) = 9999;
                    end
                elseif (temp(tooShortRR(i)) == 1) % Add previous
                    lastNon9999 = find(correctedHR(1:tooShortRR(i)-1)~=9999,1,'last');
                    correctedHR(tooShortRR(i)) = correctedHR(tooShortRR(i)) + correctedHR(lastNon9999);
                    if (lastNon9999 >= 1)
                        correctedHR(lastNon9999) = 9999;
                    end
                end
            end
        end
        % Redefining HR and HRtimes
        HR = correctedHR(correctedHR~=9999);
    else
        % End while loop
        tooShortIntervalsPresent = false;
    end
end

%% Correction of too long intervals

% Redefine robust RR series
HRrobust = runprctile(HR,2,50);

% Detecting which RR intervals are too long compared to the robust one.
temp = find(HR>1.8*HRrobust);

% Looping over all to be corrected RR intervals.
correctedHR = HR;
for i = 1:length(temp)
    splitSize = round(HR(temp(end-i+1))/HRrobust(temp(end-i+1)));
    correctedHR = [correctedHR(1:temp(end-i+1)-1); correctedHR(temp(end-i+1))/splitSize*ones(splitSize,1); correctedHR(temp(end-i+1)+1:end)];
end

HR = correctedHR;

% Detecting intervals longer than twice the overall median in seconds.
temp = find(HR>2.5*median(HR));

% Looping over all to be corrected RR intervals.
correctedHR = HR;
for i = 1:length(temp)
    splitSize = round(HR(temp(end-i+1))/(median(HR)));
    correctedHR = [correctedHR(1:temp(end-i+1)-1); correctedHR(temp(end-i+1))/splitSize*ones(splitSize,1); correctedHR(temp(end-i+1)+1:end)];
end

HR = correctedHR;

%% Redefine HRtimes

HRtimes = [HRtimes(1);HRtimes(1)+cumsum(HR)];
end

function [I] = minIndex(X,dim)

% Function that returns the index of the minimum value of the array or
% matrix X, following the input dimension. In case of multiple values
% having the same minimum value, the lowest index value is returned.
%
% Use: [I] = minIndex(X,dim)
%
% Author: Tim Willemen
% Last edit: 04-04-2013

[~,I] = min(X,[],dim);
end

