function [HRtimes] = pantom(ecg)

% Function that extracts HR from raw ecg signal, using an adaptation of the
% Pan-Tompkins algorithm for 200Hz ECG signal (p187-190, Biomedical Signal
% Analysis). The sample rate of the ecg should thus be 200 Hz. Some
% adaptations were made to make sure beginning and end peaks can be
% detected.
%
% Use: [HRtimes] = pantom(ecg)
% 
% Author: Tim Willemen
% Last edit: 26-03-2014

%% Pan-Tompkins algorithm

% Normalize ecg signal
necg = ecg/max(ecg);

% % Low-pass filter
% b = [1 0 0 0 0 0 -2 0 0 0 0 0 1]/32;
% a = [1 -2 1];
% lpecg = filter(b, a, necg);
% clear necg;
lpecg = necg;
clear necg;

% High-pass filter (All-pass - Low-pass)
% Low-pass
b = zeros(1,33); b(1) = 1; b(33) = -1;
a = [1 -1];
hlp = filter(b,a,lpecg);
% All-pass
b = zeros(1,17); b(17) = 1;
a =1;
hap = filter(b,a,lpecg);
% High-pass
hpecg = hap - hlp/32;
clear lpecg hap hlp;

% Derivative
b = [2 1 0 -1 -2]; b = b/8;
a = 1;
decg = filter(b,a,hpecg);
clear hpecg;

% Squaring
sqecg = decg.^2;
clear decg;

% Integrating (Window width=30 found to be suitable for 200Hz signal)
[b] = ones(1,30)/30;
[a] = 1;
intecg = filter(b,a,sqecg);
clear sqecg;

% Renormalize
necg = intecg/max(intecg);
clear intecg;

% Raw detection of peaks (signal raising or lowering for 15 datapoints)
temp1 = 15+find(necg(1:end-30)<=necg(16:end-15) & necg(16:end-15)>=necg(31:end));
temp2 = find(diff(temp1)>15);
peakIntervals = [[temp1(1);temp1(temp2+1)] [temp1(temp2);temp1(end)]]';
peakMax = zeros(length(peakIntervals),1);
peakPos = zeros(length(peakIntervals),1);
for i = 1:length(peakIntervals)
    [peakMax(i) peakPosPeakInterval] = max(necg(peakIntervals(1,i):peakIntervals(2,i)));
    peakPos(i) = peakIntervals(1,i)+peakPosPeakInterval-1;
end

% Adaptive tresholding of the previously found peaks
% Pick startvalues
sortedPeakMax = sort(peakMax);
SPKIstart = mean(sortedPeakMax(round(2/8*length(sortedPeakMax)):round(7/8*length(sortedPeakMax))));
SPKI = SPKIstart;
NPKI = SPKI/2;
threshold = NPKI + 0.25*(SPKI-NPKI);

% Iterate over all peaks found, selecting those above threshold, while
% adapting thresholds along the way.
counter = 1;
trueMax = [];
truePos = [];
for i = 1:length(peakMax)
    if (peakMax(i) >= threshold)
        trueMax(counter,1) = peakMax(i);
        truePos(counter,1) = peakPos(i);
        if (peakMax(i) <= 5*SPKIstart)
            SPKI = 0.125*peakMax(i)+0.875*SPKI;
            threshold = NPKI + 0.25*(SPKI-NPKI);
        end
        counter = counter+1;
    elseif (peakMax(i) >= NPKI/10)
        NPKI = 0.125*peakMax(i)+0.875*NPKI;
        SPKI = 0.05*peakMax(i)+0.95*SPKI;
        threshold = NPKI + 0.25*(SPKI-NPKI);
    end
end

% % Adding a final truePos at the end of the necg array, in case of no ecg
% % signal present in the end. This to later on detect this phase as a too
% % long RR so we can fill this phase in with generated RR data.
% if (truePos(end) ~= length(necg))
%     truePos(end+1) = length(necg);
%     trueMax(end+1) = necg(end);
% end

% There is an introduced delay of 41 samples due to the filtering. 
truePos = truePos-41;

% Detecting the position of the peaks in the real ecg signal, within an
% interval of 50 samples around the detected location. This means within
% 0.25 seconds, since it is a 200 Hz signal.
absEcg = abs(ecg);
for i = 1:length(truePos)
    [~, localMaxEcg] = max(absEcg(max([truePos(i)-24 1]):min([truePos(i)+24 length(ecg)])));
    truePos(i) = truePos(i)+localMaxEcg-25-min([0 truePos(i)-25]);
end

% Extraction of HR
HRtimes = 0.005*truePos;
