function [y] = runprctile(x,n,p)

% Function that calculates the running percentiles p of array x over a 
% window length of size 2*n+1. It uses maximum 500 MB ram, as it will split
% up larger arrays x in smaller parts. This is particularly needed when the
% size of x and/or n becomes large.
%
% Use: [y] = runprctile(x,n,p)
%
% Author: Tim Willemen
% Last edit: 06-03-2013

nn = 2*n+1;

% Adding median values in front and in back of x to also have prctile 
% values for the first and last n values.

if (size(x,1)>1)
    xx = [median(x(1:n+1))*ones(n,1);x;median(x(end-n:end))*ones(n,1)];
else
    xx = [median(x(1:n+1))*ones(1,n),x,median(x(end-n:end))*ones(1,n)];
end

% We fix the maximum allowed size in ram to 0.5 gigabyte. Our matrix will 
% have a size of nn*length(x), so as long as this is smaller than 0.5
% gigabyte divided by 8 bytes per value in our matrix, we use the following
% fast code. Else we use a slower variant which doesn't use that much ram.

if (nn*length(x) <= (0.5*1024*1024*1024/8))
    % Constructing a matrix with the rows being the different windows over
    % which the prctiles have to be calculated.
    temp = zeros(length(xx)-(nn-1),nn);
    for i = 1:nn
        temp(:,i) = xx(i:end-(nn-i));
    end
    y = prctile(temp,p,2);
else
    % Calculate the amount of necessary splits, and define begin and end
    % indices for these splits.
    numberOfSplits = ceil(length(x)/((0.5*1024*1024*1024/8)/nn));
    yy = cell(numberOfSplits,1);
    splitStarts = ((1:numberOfSplits)-1)*round(length(x)/numberOfSplits)+1;
    splitEnds = (1:numberOfSplits)*round(length(x)/numberOfSplits)+(nn-1);
    splitEnds(end) = length(xx);
    
    % Loop over the different splits, calculating the prctiles in the same
    % way as when no split was needed.
    display(['Number of splits: ' num2str(numberOfSplits)]);
    for k = 1:numberOfSplits
        tic
        zz = xx(splitStarts(k):splitEnds(k));
        temp = zeros(length(zz)-(nn-1),nn);
        for i = 1:nn
            temp(:,i) = zz(i:end-(nn-i));
        end
        yy{k} = prctile(temp,p,2);
        toc
    end
    
    % Combine different splits.
    y = zeros(length(x),length(p));
    for k = 1:numberOfSplits
        y(splitStarts(k):splitEnds(k)-(nn-1),:) = yy{k};
    end
end


